package com.javacorporate.forum.dbobj;

/*
 * ForumGroup.java
 *
 * Copyright 1999 Javacorporate Ltd.
 */

import com.javacorporate.dbobj.*;
import com.javacorporate.common.dbobj.NextNumber;
import com.javacorporate.db.*;
import java.util.Calendar;
import com.javacorporate.common.util.DateTime;
import com.javacorporate.common.util.ConfigManager;
import com.javacorporate.common.util.StringUtil;
import com.javacorporate.common.util.CacheEventListener;
import com.javacorporate.common.util.CacheEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

/**
 * ForumGroup 
 *
 * @version		$Revision: 1.4 $  $Date: 2000/06/11 16:39:25 $
 * @author		Michael Nash
 */
public class ForumGroup extends SecuredDBObject implements CacheEventListener {
		private static Hashtable securityCache = null;
		private static boolean trace = false;
		private static int maxCacheSize = 500;
		private static boolean listenersSetup = false;

	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject
	 */
	public ForumGroup() throws DBException {
		super();

		if (StringUtil.notNull(ConfigManager.getProperty("trace")).equalsIgnoreCase("Y")) {
				trace = true;
		}

		if (!listenersSetup) {
			DBObject.addCacheEventListener("com.javacorporate.common.dbobj.GroupMembers", this);
			DBObject.addCacheEventListener("com.javacorporate.forum.dbobj.ForumSecurity", this);
			listenersSetup = true;
		}
	} /* ForumGroup() */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject
	 */
	public ForumGroup(String theUser) throws DBException {

		super(theUser);
	} /* ForumGroup(String) */
	/**
	 * Extends the usual add method to fetch a next number field
	 * and set the created date/time
	 *
	 * @throws	DBException If the next number cannot be determined or
	 * 			the add fails
	 */
	public void add() throws DBException {
		if (getField("ParentGroup").equals("")) {
			setField("ParentGroup", "0");
		}
		if (getField("GroupType").equals("")) {
			setField("GroupType", "0");
		}
		NextNumber myNext = new NextNumber("SYSTEM");
		long myIdNum = myNext.getNextTentative("FGROUP");
		setField("GroupId", new String().valueOf(myIdNum));

		Calendar rightNow = Calendar.getInstance();
		setField("Created", DateTime.getDateTimeForDB());
		super.add();
		myNext.getNextConfirm("FGROUP");
	} /* add() */
/**
	 * Add a new cached item
	 */
	private synchronized void addCachedSecurity(String userName, String groupId, String methods) {

		String myName = new String(thisClass + "addCachedSecurity(String, String, String)");

		if (trace) {
			System.err.println(myName + ":Adding cached access for '"
				+ userName + "' for methods(s) '" + methods + "' group "
				+ groupId);
		}

		if (securityCache == null) {
			securityCache = new Hashtable(20);
			String cacheMax = StringUtil.notNull(ConfigManager.getProperty("forum.securitycachesize"));
			if (!cacheMax.equals("")) {
				maxCacheSize = new Integer(cacheMax).intValue();
			}
		}

		/* If there is already an entry for this user, add the given */
		/* state(s) to the list of allowed things - no dups */
		String existing = (String) securityCache.get(userName + "|" 
			+ groupId);

		if (existing != null) {
			if (trace) {
				System.err.println(myName + ":There was existing security in cache");
			}

			/* Using the two hashtable below "merges" the entries, creating */
			/* one hash with all of the states in it */
			Hashtable existingHash = new Hashtable(3);
			String oneState = null;
			StringTokenizer stk = new StringTokenizer(existing, " ");
			while (stk.hasMoreElements()) {
				oneState = (String) stk.nextToken();
				existingHash.put(oneState, oneState);
			}

			Hashtable newHash = new Hashtable(3);
			StringTokenizer stk2 = new StringTokenizer(methods, " ");
			while (stk2.hasMoreElements()) {
				oneState = (String) stk2.nextToken();
				existingHash.put(oneState, oneState);
			}

			/* Time-saver: If we have "*", don't bother with the rest */
			if (existingHash.get("*") != null) {
				existing = new String("*");
			} else {
				StringBuffer existingBuffer = new StringBuffer("");
				for (Enumeration e = existingHash.keys();
					e.hasMoreElements(); ) {
					existingBuffer.append((String) e.nextElement());
					existingBuffer.append(" ");
				}
				existing = existingBuffer.toString();
			}
			securityCache.remove(userName + "|" + groupId);
			securityCache.put(userName + "|" + groupId, existing);
			if (trace) {
				System.err.println(myName + ":Cache updated for user '"
					+ userName + "', group " + groupId
					+ ", permissions now '" + existing + "'");
			}
			return;
		} else {
			if (trace) {
				System.err.println(myName + ":There was no existing security "
					+ "in cache");
			}

			if (securityCache.size() <= maxCacheSize) {
				securityCache.put(userName + "|" + groupId, methods);
			} else {
				Enumeration e = securityCache.keys();
				String s = (String) e.nextElement();
				securityCache.remove(s);		
				securityCache.put(userName + "|" + groupId, methods);
			}
		}
	} /* addCachedSecurity */
	/**
	 * Is this function in this group allowed for the specified user?
	 *
	 * @throws	DBException 
	 */
	public boolean allowedForUser(String userName, String methodCode) throws DBException {

		String myName = new String(thisClass + "allowedForUser(String, String)");

		String allMethods = new String("AUDBM");

		if (allMethods.indexOf(methodCode) == -1) {
			throw new DBException(myName + "'" + methodCode + "' is not a valid method");
		}

		trace = true;
		
		/* if the security cache is available, use it */

		securityCache = null;
		
		if (securityCache != null) {
			String sec = (String) securityCache.get(userName + "|" + getField("GroupId"));

			/* if there is an entry for this user */
			if (sec != null) {
				if ((sec.indexOf(methodCode) != -1) || (sec.indexOf("*") != -1)) {
					if (trace) {
						System.err.println(myName + ":User '" + userName 
							+ "' allowed method '" + methodCode + "' via cache");
					}
					return true;
				} else {
					/* otherwise they are denied */
					if (trace) {
						System.err.println(myName + ":User '" 
							+ userName + "' denied method '" + methodCode
							+ "' via cache. Cache entry was '" + sec + "'");
					}
					return false;
				}
			}
		} /* if security cache is available */

		DBConnectionPool myPool = null;
		DBConnection conn = null;

		try {
			myPool = DBConnectionPool.getInstance();
			conn = myPool.getConnection("Forum Security for forum group '" + getField("GroupId") + "' and user '" + userName + "'");
			conn.execute("SELECT MethodCode FROM FORUMSECURITY, "
				+ "GROUPMEMBERS WHERE GROUPMEMBERS.UserName = '" + userName + "'"
				+ " AND GROUPMEMBERS.GroupName = FORUMSECURITY.GroupName "
				+ " AND FORUMSECURITY.GroupId = '" + getField("GroupId") + "'");

			boolean allowed = false;
			while (conn.next()) {
				/* if we get here, we know the user is allowed to access */
				/* this group. Now check the specific actions they're allowed. */
				String allowedStates = conn.getString(1);	
				if ((allowedStates.indexOf(methodCode) != -1) || (allowedStates.indexOf("*") != -1)) { 

					allowed = true;
				}
				//addCachedSecurity(userName, getField("GroupId"), allowedStates);
			} /* if any record found */

			myPool.release(conn);
			if (!allowed) {
				//addCachedSecurity(userName, getField("GroupId"), "");
			}
			return allowed;
		} catch(DBException de) {
			throw new DBException(myName + ":Unable to check forum group security", de);
		} finally {
			if (myPool != null) {
				myPool.release(conn);
			}
		}
	} /* allowedForUser(String) */
	/**
	 * Simply clear the security cache (to read new values) if there is a 
	 * change in a related security table
	 */
	public void cacheEvent(CacheEvent ce) {
		securityCache = null;
	} /* cacheEvent(CacheEvent) */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject#getThisDBObj()	 
	 *
	 * @returns	DBObject
	 * @throws	DBException
	 */
	public DBObject getThisDBObj() throws DBException {
		return new ForumGroup();
	} /* getThisDBObj() */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject
	 */
	protected synchronized void setupFields() throws DBException {
		setTargetTable("FORUMGROUP");

		setDescription("Forum Groups");

		addField("GroupId", "int", 0, false, "Group Id");
		addField("GroupName", "varchar", 30, false, "Group Name");
		addField("GroupDescription", "varchar", 80, true, "Group Description");
		addField("GroupType", "int", 0, false, "Group Type");
		addField("Created", "datetime", 0, false, "Created");
		addField("ParentGroup", "int", 0, false, "Parent Group");
		addField("LastPosting", "datetime", 0, true, "Last Posting");
		addField("LastPostingBy", "char", 30, true, "Last Posting By");
		addField("Project", "char", 10, true, "Project Code");

		addKey("GroupId");
		setReadOnly("Created");
		setReadOnly("GroupId");
		setLookupObject("ParentGroup", "com.javacorporate.forum.dbobj.ForumGroup");
	} /* setupFields() */
} /* ForumGroup */
