package com.javacorporate.forum.server;

/*
 * SendNotice.java
 *
 * Copyright 1999 Javacorporate Ltd.
 */

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import com.javacorporate.common.server.Job;
import com.javacorporate.db.DBException;
import com.javacorporate.dbobj.ValidValue;
import com.javacorporate.common.dbobj.User;
import com.javacorporate.common.dbobj.UserGroup;
import com.javacorporate.common.dbobj.GroupMembers;
import com.javacorporate.common.util.Log;
import com.javacorporate.common.dbobj.Event;
import com.javacorporate.common.dbobj.JobQueue;
import com.javacorporate.common.dbobj.JobQueueParam;
import com.javacorporate.common.dbobj.DownloadLog;
import com.javacorporate.common.html.*;
import com.javacorporate.common.util.StringUtil;
import com.javacorporate.forum.dbobj.ForumSubscribe;

/**
 * SendNotice allows an email notification to be sent to a group of 
 * users 
 *
 * @version		$Revision: 1.4 $  $Date: 2000/05/31 19:46:43 $
 * @author		Michael Nash
 */
public class SendNotice extends Job {

	private String thisClass = new String(this.getClass().getName() + ".");

	private boolean trace = true;

	public SendNotice() {
		addParameter("NoticeType", "Notice Type");
		addParameter("Subject", "Message Subject");
		addParameter("GroupName", "Group Name");
		addParameter("NoticeText", "Notice Text");
	} /* SendNotice */
	private void errorNotify(String msg) {
		String myName = new String(thisClass + "errorNotify(String)");

		try {
			Event myEvent = new Event("SYSERROR", myName + ":Unable to send notifications:" + msg
				+ " Job:" + getJobNumber() + " User:" + getUser(), false);
		} catch (Exception e) {
			Log.tryLog(myName, "Unable to send error event:" + e.getMessage(), "RED");
		}
	} /* errorNotify */
	public String getTitle() {
		return new String("Send Notices");
	}
	/**
 	 * Receive the notification request and send the emails
	 */
	public void run() {

		super.run();

   		String myName = new String(thisClass + "run()");

   		if (trace) {
	   		System.out.println(myName + ":Starts");
   		}

		try	{
			StringBuffer mailMsg = new StringBuffer("");

			JobQueue jq = getJobQueueEntry();
			String userName = jq.getField("UserName");
			String subject = jq.getParamValue("Subject");
			if (subject == null) {
				subject = new String("No Subject Specified");
			}
			String groupName = jq.getParamValue("GroupName");
			String noticeType = jq.getParamValue("NoticeType");

			StringBuffer noticeTextBuffer = new StringBuffer();
			JobQueueParam jqp = new JobQueueParam("SYSTEM");
			jqp.setField("JobNumber", jq.getField("JobNumber"));
			jqp.setField("ParamCode", "NoticeText%");

			if (trace) {
				System.out.println("Subject:" + subject + "/GroupName:" + groupName + "/Type:" + noticeType);
			}

			JobQueueParam oneJqp = null;

			for (Enumeration je = jqp.searchAndRetrieve("ParamNumber").elements(); je.hasMoreElements(); ) {

				oneJqp = (JobQueueParam) je.nextElement();
				noticeTextBuffer.append(oneJqp.getField("ParamValue"));
				noticeTextBuffer.append("\n");
			} /* for */

			String noticeText = noticeTextBuffer.toString();

			if (groupName.equals("")) {
				throw new ServletException(myName + ":Group Name/Id must not be blank");
			}

			Vector v = new Vector(10);

			/* All Members of a particular Security group */
			if (noticeType.equals("S")) {
				mailMsg.append("Sending to all users who are members of security group '" + groupName + "'\n");
				User oneUser = new User("SYSTEM");
				GroupMembers gl = new GroupMembers("SYSTEM");
				GroupMembers oneMember = null;
				gl.setField("GroupName", groupName);

				if (trace) {
					System.out.println("Sending to group " + groupName);
				}

				for (Enumeration e = gl.searchAndRetrieve().elements(); e.hasMoreElements(); ) {
	
					oneMember = (GroupMembers) e.nextElement();
					v.addElement(oneMember.getField("UserName"));
					System.out.println("Send to user "+ oneMember.getField("UserName"));
				}

				/* All Users subscribed to a particular Forum (any type of subscription) */
			} else if (noticeType.equals("F")) {
				mailMsg.append("Sending to all users subscribed to Forum " + groupName + "\n");
				
				ForumSubscribe sl = new ForumSubscribe();
				ForumSubscribe oneSubscribe = null;
				sl.setField("GroupId", groupName);
				for (Enumeration esl = sl.searchAndRetrieve().elements(); esl.hasMoreElements(); ) {
					oneSubscribe = (ForumSubscribe) esl.nextElement();
					v.addElement(oneSubscribe.getField("UserName"));
				}

				/* All Users Registered for Notices to a Forum */
			} else if (noticeType.equals("N")) {
				mailMsg.append("Sending to all users subscribed for notifications to Forum " + groupName + "\n");
				ForumSubscribe sl = new ForumSubscribe();
				ForumSubscribe oneSubscribe = null;
				sl.setField("GroupId", groupName);
				sl.setField("Notices", "Y");
				for (Enumeration esl = sl.searchAndRetrieve().elements(); esl.hasMoreElements(); ) {
					oneSubscribe = (ForumSubscribe) esl.nextElement();
					v.addElement(oneSubscribe.getField("UserName"));
				}
				
				/* All Users Registered for Daily Summaries to a Forum */
			} else if (noticeType.equals("D")) {
				mailMsg.append("Sending to all users subscribed for Daily Summaries to Forum " + groupName + "\n");
				ForumSubscribe sl = new ForumSubscribe();
				ForumSubscribe oneSubscribe = null;
				sl.setField("GroupId", groupName);
				sl.setField("Summaries", "Y");
				for (Enumeration esl = sl.searchAndRetrieve().elements(); esl.hasMoreElements(); ) {
					oneSubscribe = (ForumSubscribe) esl.nextElement();
					v.addElement(oneSubscribe.getField("UserName"));
				}
				
				/* All Users Subscribed to EMail Messages to a Forum */
			} else if (noticeType.equals("M")) {
				mailMsg.append("Sending to all users subscribed for EMail delivery to Forum " + groupName + "\n");
				ForumSubscribe sl = new ForumSubscribe();
				ForumSubscribe oneSubscribe = null;
				sl.setField("GroupId", groupName);
				sl.setField("EMail", "Y");
				for (Enumeration esl = sl.searchAndRetrieve().elements(); esl.hasMoreElements(); ) {
					oneSubscribe = (ForumSubscribe) esl.nextElement();
					v.addElement(oneSubscribe.getField("UserName"));
				}
				
			} else {
				throw new ServletException(myName + ":Unknown notice type:" + noticeType);
			}

			int noticeCount = 0;

			if (trace) {
				System.out.println("Sending to " + v.size() + " users");
			}

			User sendToUser = new User();
			for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
				sendToUser.clear();
				sendToUser.setField("UserName", (String) e.nextElement());
				if (sendToUser.find()) {
					if (trace) {
						System.out.println("Sending to " + sendToUser.getField("UserName"));
					}
					mailMsg.append("Sending notice to "
						+ sendToUser.getField("Descrip") + " ("
						+ sendToUser.getField("UserName") + ")\n");
					
					try {
						sendToUser.notify(subject, noticeText);
						noticeCount++;
					} catch (Exception me) {
						mailMsg.append("Unable to send to user '" + sendToUser.getField("UserName") + "':" + me.getMessage());
					}
				} else {
					mailMsg.append("Unable to find user '"
						+ sendToUser.getField("UserName") + " in user table");
				}
			}

			mailMsg.append("" + noticeCount + " Notifications Sent");
			mailMsg.append("\nNotice Text:\n" + noticeText);

			sendToUser.clear();
			sendToUser.setField("UserName", userName);
			sendToUser.retrieve();
			sendToUser.notify("Sent Notifications", mailMsg.toString());	

			Log.log(myName, "Job completed - removing queue entry " 
				+ getJobQueueEntry().getField("JobNumber"), "", getUser(), getJobNumber());
		} catch(Exception de) {
			de.printStackTrace(System.err);
			errorNotify(de.getMessage());		
		} finally {
			try {
				getJobQueueEntry().delete();
			} catch(DBException de2) {
				errorNotify(de2.getMessage());		
			}
		}
	} /* run() */
} /* SendNotice */
