package com.javacorporate.forum.trx;

import com.javacorporate.db.DBConnection;
import com.javacorporate.db.DBException;
import com.javacorporate.db.DBConnectionPool;
import com.javacorporate.dbobj.Schema;
import com.javacorporate.dbobj.ValidValue;
import com.javacorporate.common.dbobj.SchemaList;
import com.javacorporate.common.dbobj.JobQueue;
import com.javacorporate.common.dbobj.JobQueueParam;
import com.javacorporate.common.util.StringUtil;
import com.javacorporate.common.util.LogHandler;
import com.javacorporate.common.server.Job;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import com.javacorporate.common.trx.DBTransaction;
import com.javacorporate.common.trx.State;
import com.javacorporate.common.trx.Input;
import com.javacorporate.common.trx.Output;
import com.javacorporate.common.trx.Action;
import com.javacorporate.common.trx.TrxException;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.IOException;

/**
 * 
 */
public class SendNotice extends DBTransaction {
	private String thisClass = getClass().getName() + ".";

	/**
	 * Our constructor "declares" what states we handle
	 */
	public SendNotice() {
		super();
		State prompt = new State("prompt", "Prompt for Notice");
		addState(prompt);

		State queuejob = new State("queuejob", "Queue Job");
		queuejob.addParameter("Group");
		queuejob.addParameter("Message");
		queuejob.addParameter("Subject");
		queuejob.addParameter("NoticeType");
		addState(queuejob);
	} /* SendNotice() */
	/**
	 * Return the title of this Transaction 
	 */
	public String getTitle() {
		return new String("Send Notices");
	} /* getTitle() */
	/**
	 * Transition to a new state
	 * 
	 * @param newState The new state to transition into
	 */
	public void newState(String newState) throws TrxException {
		super.newState(newState);

		if (newState.equals("prompt")) {
			promptState();

		} else if (newState.equals("queuejob")) {
			queueJobState();

		}	

		if (!newState.equals("prompt")) {
			Action again = new Action("Start Again", getClass().getName());
			again.setName("again");
			again.addParam("state", "prompt");
			addAction(again);
		}
	} /* newState() */
	/**
	 * Prompt for information about the notices to send
	 */
	private void promptState() throws TrxException {
		String myName = new String(thisClass + "promptState()");
		
		/* ask the user to select the type of notices to be sent */
		Input noticeType = new Input();
		noticeType.setLabel("Notice Type");
		noticeType.setName("NoticeType");
		Vector v = new Vector(2);
		v.addElement(new ValidValue("S","All Members of a Security Group"));
		v.addElement(new ValidValue("F","All Users Subscribed to a Forum"));
		v.addElement(new ValidValue("N","All Users Registered for Notices to a Forum"));
		v.addElement(new ValidValue("D","All Users Registered for Daily Summaries to a Forum"));
		v.addElement(new ValidValue("M","All Users Subscribed to EMail Messages to a Forum"));

		noticeType.setValidValues(v);
		addInput(noticeType);

		Input group = new Input();
		group.setName("Group");
		group.setLabel("Group ID or Name:");
		addInput(group);

		Input subject = new Input();
		subject.setName("Subject");
		subject.setLabel("Subject:");
		addInput(subject);


		Input messageBody = new Input();
		messageBody.setName("Message");
		messageBody.setLines(10);
		messageBody.setDisplayLength(80);
		messageBody.setLabel("Message");
		addInput(messageBody);

		
		Action queuejob = new Action("Queue Job", getClass().getName());
		queuejob.setName("queuejob");
		queuejob.addParam("state", "queuejob");
		addAction(queuejob);
	} /* promptState() */
	/**
	 * Queue the job as requested
	 */
	private void queueJobState() throws TrxException {

		String myName = new String(thisClass + "queueJobState()");
		Output paramOut = new Output("Parameters");

		try {
			JobQueue jq = new JobQueue("SYSTEM");
			jq.setField("UserName", getUser());
			jq.setField("JobCode", "com.javacorporate.forum.server.SendNotice");
			jq.setField("StatusCode", "N");
			jq.add();

			JobQueueParam jqp = new JobQueueParam("SYSTEM");
			jqp.setField("JobNumber", jq.getField("JobNumber"));
			jqp.setField("ParamNumber", "1");
			jqp.setField("ParamCode", "NoticeType");
			jqp.setField("ParamValue", getParameter("NoticeType"));
			jqp.add();

			jqp = new JobQueueParam("SYSTEM");
			jqp.setField("JobNumber", jq.getField("JobNumber"));
			jqp.setField("ParamNumber", "2");
			jqp.setField("ParamCode", "GroupName");
			jqp.setField("ParamValue", getParameter("Group"));
			jqp.add();

			jqp = new JobQueueParam("SYSTEM");
			jqp.setField("JobNumber", jq.getField("JobNumber"));
			jqp.setField("ParamNumber", "3");
			jqp.setField("ParamCode", "Subject");
			jqp.setField("ParamValue", getParameter("Subject"));
			jqp.add();
			
			/* break up the notice into lines */
			int lineNum = 1;
			String oneLine = null;
			int paramNum = 4;

			String noticeText = getParameter("Message");
			JobQueueParam oneParam = new JobQueueParam();

			BufferedReader b = new BufferedReader(new StringReader(noticeText.trim()));

			oneLine = b.readLine();
			while (oneLine != null) {

				oneParam.setField("JobNumber", jq.getField("JobNumber"));
				oneParam.setField("ParamNumber", "" + paramNum);
				oneParam.setField("ParamCode", "NoticeText" + lineNum);
				oneParam.setField("ParamValue", oneLine);
				oneParam.add();
				lineNum++;
				paramNum++;
				oneLine = b.readLine();
			}
			

			
			jq.setField("StatusCode", "A");
			jq.update();

		} catch(DBException de) {
			throw new TrxException(myName + ":Unable to queue job", de);
		} catch(IOException ie) {
			throw new TrxException(myName + ":Unable to read message", ie);
		}
		
		addOutput(new Output("Job Queued"));
	} /* queueJobState() */
} /* SendNotice */
